#!/bin/bash -e
# Filename: repair-baseimage.sh
# Author:   Matt Bearup
# Details:  Due to a known bug in pam_unix, accounts with no usable password
#           are still subject to password expiration, rendering them unusable
#           after that period has lapsed. This script identifies affected accounts
#           and disables their password expiration date
#           https://bugzilla.mindrot.org/show_bug.cgi?id=1904
#           - Restart waagent if necessary
#           - Fix broken packages if necessary

# Import shared functions
source /usr/local/bin/shared-baseimage.sh

function isPasswordNeverExpire {
    acct=$1
    if [ -z "$acct" ]; then
        bail "Cannot check if account will expire because no account name was provided"
    fi
    chage -l $acct | egrep -q '^Password\s+expires\s+:\s+never'
    return $?
}

VERBOSE=false
function UnlockAccount() {
    acct=$1
    if [ -z "$acct" ]; then
        echo "Must specify a username to check its status"
        return 1
    fi
    status=$(passwd -S $acct | awk '{print $2}')
    # If account is using a "locked" password, remove expiration date
    if [[ "$status" == "L" ]] && ! isPasswordNeverExpire $acct; then
        $VERBOSE && echo "DISABLING password expiration for $acct"
        chage -E -1 -M -1 -d -1 $acct 2>&1 >> /dev/null
        return 0
    else
        $VERBOSE && echo "NO CHANGE to password expiration policy for $acct"
    fi
    return 1
}

while (( "$#" )); do
    if [ "$1" == "-v" ]; then
        VERBOSE=true
    fi
    shift
done

# Restart waagent if it's stopped
if [ -e /usr/sbin/waagent ]; then
    # waagent is installed. Restart if pid file doesn't exist, or pid does not exist
    if [ ! -e /var/run/waagent.pid ] || ! ps -p $(head -n 1 /var/run/waagent.pid); then
        $VERBOSE && echo "Waagent is dead - restarting"
        service walinuxagent restart
    fi
fi

# Unlock accounts if needed
if [ ! -e /etc/passwd ]; then
    echo "/etc/password not found on system!"
    exit 1
fi

for acct in $(cat /etc/passwd | cut -d ":" -f 1); do
    if UnlockAccount $acct; then
        accounts+=" $acct"
    fi
done
if [ -n "$accounts" ]; then
    echo "Disabled password expiration for:$accounts"
    logger -p "auth.info" -t UnlockAccounts -- "Disabled password expiration for: $accounts"
fi

# Repair broken packages if necessary
if ! apt-get check -qq; then
    dpkg --configure -a > /dev/null || logger -p "syslog.err" -t repair-baseimage -- "Failed to fix broken packages via dpkg --configure -a"
fi

# Repair /etc/fstab if necessary
if ! TryFixEphemeralDriveMapping /etc/fstab; then
    bail "Failed to fix ephemeral drive mapping in /etc/fstab"
fi

# Repair serial console output if it was ever disabled
if grep '^GRUB_CMDLINE_LINUX_DEFAULT=' /etc/default/grub |grep -v 'console=ttyS0'; then
    line=$(grep '^GRUB_CMDLINE_LINUX_DEFAULT=' /etc/default/grub)
    [ -n "$line" ] || bail "Failed to fix serial console output"
    line=$(echo $line | sed 's/"$/ console=ttyS0"/')
    sed -i -e "/^GRUB_CMDLINE_LINUX_DEFAULT=/ c $line" /etc/default/grub
    update-grub
fi
